import java.io.BufferedReader;
import java.io.FileReader;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.IOException;
import java.util.regex.Pattern;
import java.util.regex.Matcher;
import java.util.regex.PatternSyntaxException;

public class Grep1 {
    private Pattern pattern;
    private Matcher matcher;

    private String path;

    public Grep1(String regex) throws PatternSyntaxException {
        pattern = Pattern.compile(regex);
    }

    public void grep(BufferedReader reader) throws IOException {
        int lineNumber = 0;

        while (reader.ready()) {
            String line = reader.readLine();

            if (line == null) {
                return;
            }
            
            lineNumber++;
            grep(line, lineNumber);
        }
    }

    public void grep(String input, int lineNumber) {
        try {
            if (matcher == null) {
                matcher = pattern.matcher(input);
            } else {
                matcher.reset(input);
            }

            if (matcher.find()) {
                String path = getFilePath();
                if (path != null) {
                    System.out.print(path + ":");
                }
                System.out.println(lineNumber + ":" + input);
            }

        } catch (PatternSyntaxException ex) {
            System.err.println(ex.getMessage());
            ex.printStackTrace();
            return;
        }
    }

    public void setFilePath(String path) {
        this.path = path;
    }

    public void resetFilePath() {
        this.path = null;
    }

    public String getFilePath() {
        return path;
    }


    public static void main(String[] args) {
        if (args.length < 1) {
            System.out.println("Usage: java Grep1 pattern [inputfile] ...");
            return;
        }

        Grep1 grepper = null;
        try {
            grepper = new Grep1(args[0]);
        } catch (PatternSyntaxException ex) {
            System.err.println(ex.getMessage());
            ex.printStackTrace();
            return;
        }

        try {
            BufferedReader reader = null;
            if (args.length == 1) {
                reader = new BufferedReader(new InputStreamReader(System.in));
                grepper.resetFilePath();
                grepper.grep(reader);
                reader.close();
            } else if (args.length > 1) {

                for (int i = 1 ; i < args.length ; i++) {
                    reader = new BufferedReader(new FileReader(args[i]));
                    grepper.setFilePath(args[i]);
                    grepper.grep(reader);
                    reader.close();
                }
            }

        } catch(IOException ex) {
            ex.printStackTrace();
        }
    }
}

