import java.io.FileReader;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.IOException;
import java.util.regex.Pattern;
import java.util.regex.Matcher;
import java.util.regex.PatternSyntaxException;

public class Grep2 {
    private Pattern pattern;
    private Pattern linePattern;
    private Matcher matcher;

    private String path;

    public Grep2(String regex) throws PatternSyntaxException {
        pattern = Pattern.compile(regex);
        linePattern = Pattern.compile(".*\r?\n");
    }

    public String readContents(Reader reader) throws IOException {
        char[] chars = new char[10000];
        StringBuffer buffer = new StringBuffer();

        while (true) {
            int size = reader.read(chars);
            if (size < 0) {
                break;
            }
            
            buffer.append(chars, 0, size);
        }
        
        return buffer.toString();
    }

    public void grep(Reader reader) throws IOException {

        String contents = readContents(reader);

        int lineNumber = 0;
        Matcher lineMatcher = linePattern.matcher(contents);

        while (lineMatcher.find()) {
            lineNumber++;
            String line = lineMatcher.group();

            grep(line, lineNumber);
        }
    }

    public void grep(String input, int lineNumber) {
        try {
            if (matcher == null) {
                matcher = pattern.matcher(input);
            } else {
                matcher.reset(input);
            }

            if (matcher.find()) {
                String path = getFilePath();
                if (path != null){
                    System.out.print(path + ":");
                }
                System.out.print(lineNumber + " : " + input);
            }

        } catch (PatternSyntaxException ex) {
            System.err.println(ex.getMessage());
            ex.printStackTrace();
            return;
        }
    }

    public void setFilePath(String path) {
        this.path = path;
    }

    public void resetFilePath() {
        this.path = null;
    }

    public String getFilePath() {
        return path;
    }


    public static void main(String[] args) {
        if (args.length < 1) {
            System.out.println("Usage: java Grep1 pattern [inputfile] ...");
            return;
        }

        Grep2 grepper = null;
        try {
            grepper = new Grep2(args[0]);
        } catch (PatternSyntaxException ex) {
            System.err.println(ex.getMessage());
            ex.printStackTrace();
            return;
        }

        try {
            Reader reader = null;
            if (args.length == 1) {
                reader = new InputStreamReader(System.in);
                grepper.resetFilePath();
                grepper.grep(reader);
                reader.close();
            } else if (args.length > 1) {

                for (int i = 1 ; i < args.length ; i++) {
                    reader = new FileReader(args[i]);
                    grepper.setFilePath(args[i]);
                    grepper.grep(reader);
                    reader.close();
                }
            }

        } catch(IOException ex) {
            ex.printStackTrace();
        }
    }
}

